!
! Copyright (C) 1991-2003  ; All Rights Reserved ; ATMET, LLC
! 
! This file is free software; you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software 
! Foundation; either version 2 of the License, or (at your option) any later version.
! 
! This software is distributed in the hope that it will be useful, but WITHOUT ANY 
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
! PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with this 
! program; if not, write to the Free Software Foundation, Inc., 
! 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!======================================================================================

SUBROUTINE read_rawi(iun,olat1,olat2,olon1,olon2)

implicit none

integer :: iun
real :: olat1,olat2,olon1,olon2

include 'prep_com.h'

real :: p,t,z,h,d,f,zdf
common/snding/p(maxrlev),t(maxrlev),z(maxrlev),h(maxrlev)  &
     ,d(maxrlev),f(maxrlev),zdf(maxrlev)

real :: pp,zp,tp,hp,vz,uz,zz,rp,dz,fz
common/snding2/  &
     pp(maxrlev),zp(maxrlev),tp(maxrlev),hp(maxrlev)  &
     ,vz(maxrlev),uz(maxrlev),zz(maxrlev),rp(maxrlev)  &
     ,dz(maxrlev),fz(maxrlev)

real :: tsz(maxrlev)
integer :: iflags(3,5),imarker,iver_raw,nobs,lp,lz,n,k,nlv,lbc,nsta  &
          ,iyr,i,imo,idy,ihr
real :: xlat,xlon,elev,p00,rcp,cp,g,wfact,bchyd,pio,zso,tho
real, external :: rs

real, parameter :: rmiss=-999.


character(len=8) :: idsta

!-----------------------------------------------------------------
!  Open the rawindsonde input file.
!-----------------------------------------------------------------
PRINT 559,INRAWI
559  FORMAT(/'   Acquiring sounding input file    ',A40)
OPEN(3,file=INRAWI,status='old',form='FORMATTED')

! beg write ralph2 file header for rawin:
imarker=999999
iver_raw=2
write(iun,'(i6,1x,i1)') imarker,iver_raw
! end write ralph2 file header for rawin:

!-----------------------------------------------------------------
!  Start reading the soundings. Modify the following code to read
!    your input rawindonde file. You may use the arrays in the
!    common/snding/ above as scratch arrays if necessary.
!-----------------------------------------------------------------

do nobs=1,10000
   read(3,*,end=90,err=90) lp,lz,xlat,xlon,elev
   read(3,*,end=90,err=90)(P(N),z(n),T(N),H(N),N=1,lp)
   read(3,*,end=90,err=90)(zdf(N),f(n),d(n),n=1,lz)

!-----------------------------------------------------------------
!  Optionally check the rawindsonde location to see if it falls
!    in a smaller domain.
!-----------------------------------------------------------------
   IF(XLAT.GT.olat2.OR.XLAT.LT.olat1)GO TO 10
   IF(XLON.GT.olon2.OR.XLON.LT.olon1)GO TO 10

!-----------------------------------------------------------------
!  In the following section, modify the code to convert the input
!    variables to
!    SI units and appropriate form and fill up the following
!    variables and arrays. Set missing values to -999.
!
!         PP - Pressure (Pascals)
!         TP - Temperature (Kelvin)
!         RP - Relative humidity (fraction)
!         ZP - Height of pressure levels (meters MSL)
!         ZZ - Height of wind levels (meters MSL)
!         DZ - Wind direction (degrees)
!         FZ - Wind speed (m/s)
!         XLAT - Station latitude (positive north, negative south)
!         XLON - Station longitude (positive east, negative west)
!         ELEV - Station elevation (meters MSL)
!         LP   - Number of pressure levels
!         LZ   - Number of wind levels in height
!         IDSTA- Station ID
!-----------------------------------------------------------------


   p00=100000.
   rcp=.286
   cp=1004.
   wfact=.5144
   g=9.8

   LP=0
   LZ=0

   DO K=1,NLV
      PP(K)=-999.
      ZP(K)=-999.
      TP(K)=-999.
      RP(K)=-999.
      IF(T(K).GT.998.0.OR.P(K).GT.9998.0) GO TO 20

      LP=LP+1
      ZP(LP)=Z(K)
      PP(LP)=P(K)*100.
      TP(LP)=T(K)+273.16
      IF(H(K).LT.99.)THEN
         RP(LP)=RS(PP(LP),TP(LP)-H(K))/RS(PP(LP),TP(LP))
         tsz(lp)=tp(lp)*(1.+.61*rs(pp(lp),tp(lp))*rp(lp))  &
              *(p00/pp(lp))**rcp
      ELSE
         RP(LP)=-999.
         tsz(lp)=tp(lp)*(p00/pp(lp))**rcp
      ENDIF
!
20         CONTINUE
   ENDDO

!-----------------------------------------------------------------
!        Some soundings do not report heights at the
!        significant levels, so the following will hydrostatically
!        recompute the entire sounding heights. Delete this section
!        if your heights are adequately defined.


!        Find a boundary condition as first level at or below 500 mb

   bchyd=50000.
   do k=lp,1,-1
      if(pp(k).ge.bchyd.and.tp(k).lt.1e19.and.zp(k).lt.99998.)  &
           then
         lbc=k
         go to 52
      endif
   enddo
   print*,' Could not find good rawind z  boundary level-',nsta+1
   go to 10
52      continue


   pio=cp*(pp(lbc)/p00)**rcp
   zso=zp(lbc)
   tho=tsz(lbc)
   do k=lbc+1,lp
      zp(k)=-999.
      if(pp(k)+tsz(k).lt.1e19)then
         zp(k)=zso+(tsz(k)+tho)*.5/g  &
              *(pio-cp*(pp(k)/p00)**rcp)
         zso=zp(k)
         pio=cp*(pp(k)/p00)**rcp
         tho=tsz(k)
      endif
   enddo

   zso=zp(lbc)
   pio=cp*(pp(lbc)/p00)**rcp
   tho=tsz(lbc)
   do k=lbc-1,1,-1
      zp(k)=-999.
      if(pp(k)+tsz(k).lt.1e19)then
         zp(k)=zso+(tsz(k)+tho)*.5/g  &
              *(pio-cp*(pp(k)/p00)**rcp)
         zso=zp(k)
         pio=cp*(pp(k)/p00)**rcp
         tho=tsz(k)
      endif
   enddo
!-----------------------------------------------------------------

   do k=1,lz
      if(zdf(k).lt.99999.0.and.d(k).lt.999.0  &
           .and.f(k).lt.999.0)then
         zz(lz)=z(k)
         dz(lz)=d(k)
         fz(lz)=f(k)*wfact
      else
         zdf(k)=-999.
         dz(k)=-999.
         fz(k)=-999.
      endif
   enddo

!-----------------------------------------------------------------
!-----------------------------------------------------------------
!  Write out the rawindsonde to the opened file in the "standard"
!     RAMS format.

! beg write ralph2 data for rawin:

   write(iun,530) iyr+1900,imo,idy,ihr,idsta,lp,lz,xlat,xlon,elev

530      format(i4.4,2i3,3x,i4.4,3x,a8,5x,i4.4,2x,i4.4,3(2x,f10.5))

    do k=1,lp
      call setflags_f(pp(k),rmiss,iflags(1,1))
      call setflags_f(zp(k),rmiss,iflags(1,2))
      call setflags_f(tp(k),rmiss,iflags(1,3))
      call setflags_f(rp(k),rmiss,iflags(1,4))
      write(iun,80)pp(k)       ,(iflags(i,1),i=1,3)  &
                  ,zp(k)       ,(iflags(i,2),i=1,3)  &
                  ,tp(k)-273.16,(iflags(i,3),i=1,3)  &
                  ,rp(k)       ,(iflags(i,4),i=1,3)
80         format(f12.3,2x,3i1  &
,            f12.3,2x,3i1  &
,            f10.2,2x,3i1  &
,            f10.4,2x,3i1)
   enddo

   do k=1,lz
      call setflags_f(zz(k),rmiss,iflags(1,1))
      call setflags_f(fz(k),rmiss,iflags(1,2))
      call setflags_f(dz(k),rmiss,iflags(1,3))
      write(iun,85) zz(k),(iflags(i,1),i=1,3)  &
                   ,fz(k),(iflags(i,2),i=1,3)  &
                   ,dz(k),(iflags(i,3),i=1,3)
85         format(f12.3,2x,3i1  &
,            f10.2,2x,3i1  &
,            f10.2,2x,3i1)
   enddo

! end write ralph2 data for rawin

!-----------------------------------------------------------------
10      continue
enddo

90 CONTINUE

close(3)

RETURN
END

!-----------------------------------------------------------------------

subroutine setflags_f(var,rmiss,iflag)

integer :: iflag(3)
real :: var, rmiss

integer :: i

if(var.eq.rmiss)then
  do i=1,3
    iflag(i)=9
  enddo
else
  do i=1,3
    iflag(i)=0
  enddo
endif

return
end
